/*
random number generator - four-tap register-shift generator
^^^^^^^^^^^^^^^^^^^^^^^   good quality
                          grid of rnd() is at least 2^-53 (as fine as possible)

initialization: rndinit(tablen,seed)
where tablen = irrelevant
      seed = any integer (0 uses seed from time)

RNDMETHOD:
  rnd():
  1 = can be exactly 1 (probability ~ 4e-17), fastest
  2 = cannot be 1, IEEE numbers assumed
  3 = cannot be 1, general (but slowest)
  rndcos():
  1 = can be exactly +1 or -1 (probability ~ 1e-16), fastest
  2 = the same as 3
  3 = cannot be +1 nor -1, general
*/
#ifndef RNDMETHOD 
#define RNDMETHOD 3
#endif

#include <stdlib.h>
#include <math.h> /* fmod() */
#include <time.h>

void extError(const char *msg, const char *f, int l);
#define Error(X) extError(X,__FILE__,__LINE__)

#include "rndgen.h" /* instead of rndgen0.h */

#define A 471  
#define B 1586 
#define C 6988 
#define D 9689 
#define M 16383
#define trnd (++nd,ra[nd&M]=ra[(nd-A)&M]^ra[(nd-B)&M]^ra[(nd-C)&M]^ra[(nd-D)&M])

static unsigned4 ra[M+1],nd;

unsigned4 rndinit(int4 tablen,int4 seed)
{
 time_t t;
 static unsigned4 startseed;
#define irnd000 (unsigned)((startseed*=78125U)/0x10000U)

 if (seed==0) { time(&t); seed=t; }
 startseed=seed<<1 | 1;

 for (nd=0; nd<=M; nd++) {
   unsigned4 u=irnd000;
   ra[nd]=irnd000 | (u<<16); }

 return 0xffffffffU;
}

unsigned4 Irnd(void)
{
 return trnd;
}

double rnd(void)
/* random number in [0,1) (unless RNDMETHOD==1) */
{
#if RNDMETHOD==1
 double r=trnd*(1./4294967296e0);
 return (r+(double)trnd)*(1./4294967296e0);
#elif RNDMETHOD==2
 double r=(trnd&0xfffff800U)*(1./4294967296e0);
 return (r+(double)trnd)*(1./4294967296e0);
#elif RNDMETHOD==3
 double rr,r;
 do {
   r=trnd*(1./4294967296e0);
   rr=(r+trnd)*(1./4294967296e0);
   } while (rr==1e0) ;
 return rr;
#elif
#error bad RNDMETHOD: use 1,2,3
#endif
}

double rndcos(void)
/* random number in [-1,1) */
{
#if RNDMETHOD==1
 double r=trnd*(1./4294967296e0);
 return (r+trnd)*(1./2147483648e0)-1;
#elif RNDMETHOD==2 || RNDMETHOD==3
 double rr,r;
 do {
   r=trnd*(1./4294967296e0);
   rr=(r+trnd)*(1./2147483648e0)-1;
   } while (fabs(rr)==1e0) ;
 return rr;
#elif
#error bad RNDMETHOD: use 1,2,3
#endif
}

#undef A
#undef B
#undef C
#undef D
#undef M

#include "rndetc.c"
