/*
random number generator - four-tap register-shift generator
^^^^^^^^^^^^^^^^^^^^^^^   grid of rnd() is at least 2^-53 (as fine as possible)
                          high quality and fast

initialization: rndinit(tablen,seed)
where tablen = irrelevant
      seed = any integer (0 uses seed from time)

WARNING: rnd() and rndcos() may return interval ends
irnd is based on 32 bits
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h> /* fmod() */
#include <time.h>

void extError(const char *msg, const char *f, int l);
#define Error(X) extError(X,__FILE__,__LINE__)

#include "rndgen.h" /* instead of rndgen0.h */

#define A 471  
#define B 1586 
#define C 6988 
#define D 9689 
#define M 16383
#define trnd \
  (++nd,ra[nd&M]=ra[(nd-A)&M]^ra[(nd-B)&M]^ra[(nd-C)&M]^ra[(nd-D)&M])

static unsigned4 ra[M+1],nd;

#include "rndlong.c"

char rndinfo[80]="rndgen4: 2*32 bit 4-tap register-shift, grid<=1.11e-16";

unsigned4 rndinit(int4 tablen,int4 seed)
{
 time_t t;
 static unsigned4 startseed;
#define irnd000 (unsigned)((startseed*=78125U)/0x10000U)

 if (sizeof(long long)!=8) Error("sizeof(long long)<>8");
 if (sizeof(int)!=4) Error("sizeof(int)<>4");

 if (seed==0) { time(&t); seed=t; }
 sprintf(rndinfo+54," seed=%d",seed);

 startseed=seed<<1 | 1;

 for (nd=0; nd<=M; nd++) {
   unsigned4 u=irnd000;
   ra[nd]=irnd000 | (u<<16); }

 setlonglong();

 return 0xffffffffU;
}

unsigned4 Irnd(void)
{
 return trnd;
}

unsigned4 irnd(unsigned N)
{
  return ((long long unsigned)(trnd)*N)>>32;
}

double rnd(void)
/* random number in [0,1] */
{
  unsigned u1=trnd;

  return (double)((long long unsigned)u1<<32 | trnd)*MPL;
}

double rndcos(void)
/* random number in [-1,1] */
{
  unsigned u1=trnd;

  return (double)(long long int)((long long unsigned)u1<<32 | trnd)*MPLCOS;
}

#undef A
#undef B
#undef C
#undef D
#undef M

#include "rndetc.c"
