/* files of variable record lengths - C version of VARFILE.PAS
                         (c) J.Kolafa 1990

NOTE: quite obsolete piece of software kept here for compatibility reasons
11/2001: VarError() and GetSize() made static to improve style

1 record requires (2*sizeof(int4)+size) bytes 
max length of a record is given by int4
record format: < int4 size >< size bytes of data >< int4 checksum >
size==0 denotes EOF 
#ifdef Reverse then the order of bytes in int4 ints is reversed
  (sizeof(int4) must be 2 or 4)
NOTE VarGet no longer supported.
Since VarSize is known before the variable is read, you can use
alloc(XXX,VarSize); VarRead(XXX,VarSize);  
instead
*/

#include "ground.h"

#define SDS
#include "alloc.h"

#include "varfile.h"

#define VarSum0 0x4A4B;

/* global: */
FILE *VarFile;
int4 VarSize; /* size of last written/read variable */
char VarEOF=1;  /* EOF flag */

/* local: */
static char VarOpenW;
static char VarErrStr[128];
static char *VarErrEnd=VarErrStr;

static void VarError(const char *msg,char *file,int line)
{
strcat(VarErrStr+strlen(VarErrStr),msg);
extError(VarErrStr,file,line);
}
#undef Error
#define Error(X) VarError(X,__FILE__,__LINE__)

#ifdef Reverse
int VarReverse=0; /* must be set to activate endian change */

void ReverseOrder(void *a,int n)
/*** reverse order of n bytes pointed by a ***/
{
char c,*b; int i;
if (!VarReverse) return;
b=(char *)a;
loop (i,0,n/2) { c=b[i]; b[i]=b[n-1-i]; b[n-1-i]=c; }
}

void VarReverseOrder(int4 *a)
{
ReverseOrder(a,sizeof(int4));
}
#endif

static void GetSize(void)
{
if (fread(&VarSize,sizeof(VarSize),1,VarFile)!=1) Error("size read");
VarReverseOrder(&VarSize);
VarEOF = VarSize==0;
}


void VarOpen(char *fn, char *mode)
{
char m[3];

sprintf(VarErrStr,"%sing \"%s\":",
	*mode=='r' ? "read" : *mode=='w' ? "writ" : "?-",
	fn);
VarErrEnd=VarErrStr+strlen(VarErrStr);
m[0] = *mode; m[1] = 'b'; m[2] = 0;
if (!(VarFile=fopen(fn,m))) Error("open");
VarOpenW = m[0]=='w';
if ( ! (VarOpenW = m[0]=='w') ) GetSize();
}


void VarPut(void *v, int4 size)
{
int4 i,s;
unsigned char a;

if (!size) Error("0 record to write");
VarReverseOrder(&size);
fwrite(&size,sizeof(size),1,VarFile);
VarReverseOrder(&size);
s = VarSum0;
for (i = 0; i<size; i++) {
  a = *((unsigned char*)v+i); s += a; putc(a,VarFile); };
VarReverseOrder(&s);
if (fwrite(&s,sizeof(s),1,VarFile)!=1) Error("write");
}


void VarReadN(char *name,void *v, int4 size)
{
unsigned char a;
int4 i,s;
sprintf(VarErrEnd,"\nVarRead(%s,%d)",name,size);
if (VarEOF) {
  Error("unexpected EOF"); return; }
if (size != VarSize) {
  sprintf(VarErrEnd+strlen(VarErrEnd),", record length=%d",VarSize);
  Error(""); }
s = VarSum0;
for (i = 0; i<VarSize; i++) {
  a = getc(VarFile); *((unsigned char*)v+i) = a; s += a; };
if (fread(&i,sizeof(i),1,VarFile)!=1) Error("read");
VarReverseOrder(&i);
if (i!=s) Error("checksum");
GetSize();
*VarErrEnd=0;
}

#if 0
void *VarGet(void)
{
void *v;
if (v = (void *)malloc(VarSize)) { VarRead(v,VarSize); return v; }
else Error("allocate");
return v;
}
#endif

void VarClose(void)
{
int4 z=0;
if (VarOpenW) {
  if (fwrite(&z,sizeof(z),1,VarFile)!=1) Error("write EOF"); }
if (fclose(VarFile)) Error("close");
}

int4 KeyClose(int4 key)
{
int4 z=0;
if (VarOpenW) {
  if (fwrite(&z,sizeof(z),1,VarFile)!=1) Error("write EOF");
  if (fwrite(&key,sizeof(key),1,VarFile)!=1) Error("write key"); }
else {
  if (key) if (fread(&key,sizeof(key),1,VarFile)!=1) Error("read key"); }
if (fclose(VarFile)) Error("close");
return key;
}

void VarPutSds(void *s)
{
VarPut((char*)s+sizeof(int4),sdssize(s)-sizeof(int4));
}

void *VarGetSds(void)
{
char *VarSds;
sdsalloc(VarSds,VarSize+sizeof(int4));
VarRead(VarSds+sizeof(int4),VarSize);
return VarSds;
}

void VarReadSds(void *s)
{
VarRead((char*)s+sizeof(int4),sdssize(s)-sizeof(int4));
}

