/* cc -o hseos hseos.c -lm
 */
#include "../gen/include.h"
#include "../gen/powi.c"

#include "enscorr.c"

double vircorr(double rho)
{
  double eta=PI/6*rho;
  return 4*eta-2*powi(eta,2)+6.1886102971402*powi(eta,3)-1.1006765028287*powi(eta,4)+2.8083001712775*powi(eta,5)+4.86*powi(eta,6);
  /* last term +- 1.9 */
}

double drhodP(z_f Z,double rho)
{
 double d=1e-5,rp=rho+d,rm=rho-d;
 double zp=Z(PI/6*rp);
 double zm=Z(PI/6*rm);

 return (2*d)/(zp*rp-zm*rm);
}

double zJK(double eta)
{
return (1+eta+eta*eta-(1+eta)*Cub(eta)/1.5)/Cub(1-eta);
}

double zparm(double eta)
{
return (1+eta*(1+eta*(1-eta*(0.764314+eta*(0.151532+eta*0.654551)))))/Cub(1-eta);
}

double zMV(double eta)
{
 return 
  (1+eta*(1.056+eta*(1.6539+eta*0.3262))) /
  (1+eta*(0.056+eta*(0.5979+eta*0.3076))) / Cub(1-eta);
}

double zVeverka(double eta)
{
 return (1-1.577289213558097*eta)
/
(1+eta*(-2.577289213558097
  +eta*( 1.577289213558097
  +eta*( 1.635200000000000
  +eta*(-3.979683322010201
  +eta*(  .995811062924344
  +eta*( 4.544003551203230
  +eta* -4.295092248449063)))))))/Cub(1-eta);
}

double zB6_11348(double eta)
{
  double x=eta/(1-eta);
  return 1+4*x+6*powi(x,2)+2.36476838*powi(x,3)-0.869855149*powi(x,4)+1.1062803*powi(x,5)-1.09504933*powi(x,6)+0.637614043*powi(x,7)-0.227939731*powi(x,10)+0.109894767*powi(x,14)-0.00906797261*powi(x,22);
}

double drho_dPJK(double rho) /* for HS, based on Kolafa EOS */
{
 double y=PI/6*rho;
 return Pow4(1-y)*3/(((((4*y-8)*y-8)*y+12)*y+12)*y+3);
}

double drho_dPparm(double rho)
{
 double d=5e-6;
 return 2*d/(zparm(PI/6*(rho+d))*(rho+d)-zparm(PI/6*(rho-d))*(rho-d));
}

double drho_dPMV(double rho)
{
 double d=5e-6;
 return 2*d/(zMV(PI/6*(rho+d))*(rho+d)-zMV(PI/6*(rho-d))*(rho-d));
}

int main(int narg,char **arg)
{
 double rho,eta,tau;

 if (narg<3) {
   fprintf(stderr,"EOS of pure HS. Call by:\n\
  hseos {r|d|n}   rho\t(rho=number density)\n\
  hseos {t|v}     tau\t(tau=V/Vfcc, where Vfcc is close-packed fcc volume)\n\
  hseos {z}       z\t(z=Vfcc/V=1/tau)\n\
  hseos {p|y|h|e} eta\t(eta=packing fraction)\n");
   exit(0); }

 switch(arg[1][0]) {
   case 'n':
   case 'd':
   case 'r': rho=atof(arg[2]); eta=PI/6*rho; tau=sqrt(2.0)/rho; break;
   case 'y':
   case 'h':
   case 'p':
   case 'e': eta=atof(arg[2]); rho=6/PI*eta; tau=sqrt(2.0)/rho; break;
   case 't':
   case 'v': tau=atof(arg[2]); rho=sqrt(2.0)/tau; eta=PI/6*rho; break;
   case 'z': tau=1/atof(arg[2]); rho=sqrt(2.0)/tau; eta=PI/6*rho; break;
   default: Error("wrong key"); }

 put3(rho,eta,tau)
 printf("           CSK       JK/parm     Mal/Vev  Veverka173  zB6_11348\n");
 printf("      p= %10.7f %10.7f %10.7f %10.7f %10.7f\n",
	zJK(eta)*rho,zparm(eta)*rho,zMV(eta)*rho,zVeverka(eta)*rho,zB6_11348(eta)*rho);
 printf("      Z= %10.7f %10.7f %10.7f %10.7f %10.7f\n",
	zJK(eta),zparm(eta),zMV(eta),zVeverka(eta),zB6_11348(eta));
 printf("drho/dP= %10.7f %10.7f %10.7f %10.7f %10.7f\n",
	drho_dPJK(rho),drhodP(zparm,rho),drhodP(zMV,rho),drhodP(zVeverka,rho),drhodP(zB6_11348,rho));
 printf("enscorr= %10.7f %10.7f %10.7f %10.7f %10.7f",
	corr(zJK,rho),corr(zparm,rho),corr(zMV,rho),corr(zVeverka,rho),corr(zB6_11348,rho));

 /* virial exp. upto B7 */
 printf(" %10.7f\n",vircorr(rho));
return 0;
}
