real collisiontime(struct HS_s *A,struct HS_s *B) /********** collisiontime */
/*
  returns the collision time of spheres *A and *B since the current time
*/
{
 int k;
 vector dr,dv;
 real rr=0,rv=0,vv=0,D;
#ifdef DEBUG
 real tt;
#endif

 if (t!=A->t) advance(A);
 if (t!=B->t) advance(B);

#ifdef DEBUG
 if (A->t!=t) ERROR(("A->t=%g t=%g",A->t,t))
 if (B->t!=t) ERROR(("B->t=%g t=%g",B->t,t))
#endif

#if 1 /* split rr>cq test: slightly faster */
 loop (k,0,DIM) {
   dr[k]=B->r[k]-A->r[k];
   if (dr[k]>Lh) dr[k]-=L; else if (dr[k]<-Lh) dr[k]+=L;
   rr+=dr[k]*dr[k];
   if (rr>cq) return NOTIME; }
 loop (k,0,DIM) {
   dv[k]=B->v[k]-A->v[k];
   rv+=dv[k]*dr[k];
   vv+=dv[k]*dv[k]; }
#else
 loop (k,0,DIM) {
   dr[k]=B->r[k]-A->r[k];
   if (dr[k]>Lh) dr[k]-=L; else if (dr[k]<-Lh) dr[k]+=L;
   rr+=dr[k]*dr[k];
   if (rr>cq) return NOTIME;
   dv[k]=B->v[k]-A->v[k];
   rv+=dv[k]*dr[k];
   vv+=dv[k]*dv[k]; }
#endif

 if (rv>0) return NOTIME;

 D=Sqr(rv)+(sigmaq-rr)*vv;

 if (D<=0) return NOTIME;

#ifdef DEBUG
 tt=(-rv-sqrt(D))/vv;
 if (tt<0) ERROR(("%g<0",tt))
 return tt;
#else
 return (-rv-sqrt(D))/vv;
#endif
}

void collision(struct HS_s *A,struct HS_s *B) /****************** collision */
/*
  performs actual collision of spheres *A and *B
  both spheres must be at time t (see advance()) of the collision
*/
{
 int k;
 vector dr,dv,Dv;
#ifdef DEBUG
 real rr=0;
#endif
 real rv=0,D;

 loop (k,0,DIM) {
   dr[k]=B->r[k]-A->r[k];
   if (dr[k]>Lh) dr[k]-=L; else if (dr[k]<-Lh) dr[k]+=L;
#ifdef DEBUG
   rr+=dr[k]*dr[k];
#endif
   dv[k]=B->v[k]-A->v[k];
   rv+=dv[k]*dr[k]; }

#ifdef DEBUG
 if (fabs(rr/sigmaq-1)>DEBUG)
   ERROR(("%d %d rr=%.15g (%.15g expected, rel.err.=%.2g)\n\
*** A->t=%g B->t=%g t=%g collisiontime=%g",
          A-cfg,B-cfg,rr,sigmaq,rr/sigmaq-1,
	  A->t,B->t,t,collisiontime(A,B))) 
#endif

 collisions.virial+=rv;

#if 0
 /* cos (impact angle).  Use:
fgrep "~" test.prt | histogr -1 0 1/100 | plot "[-1:0][0:2]" -:A:A/-.5:- :A:B:o
 */
 prt("%.7f ~",rv/(sigma*sqrt(SQR(dv))));
#endif

 D=rv/sigmaq;
 VV(Dv,=D*dr)
 VV(A->v,+=Dv)
 VV(B->v,-=Dv)
}

