/* 
  #include file for hsmd.c with file load/save operations 
  versions: PS, SWPERT
*/

void saveall(char *fn) /******************************************* saveall */
/* 
  write the configuration, pending non-collision events and rdf
*/
{
  FILE *f;
  int i,it;
  real stop=-1;
  time_t ttt;

  syncHS();

  f=fopen(fn,"wb");
  fwrite(&N,sizeof(N),1,f);
  fwrite(&L,sizeof(L),1,f);
  fwrite(&t,sizeof(t),1,f);
  fwrite(&toffset,sizeof(toffset),1,f);
  fwrite(&sigma,sizeof(sigma),1,f);
#ifdef PS
  fwrite(&novl,sizeof(novl),1,f);
  fwrite(&vunit,sizeof(vunit),1,f);
#endif 

  loop (i,0,N) {
    if (1!=fwrite(&cfg[i].r,sizeof(vector),1,f)) ERROR(("%s: write",fn))
    if (1!=fwrite(&cfg[i].v,sizeof(vector),1,f)) ERROR(("%s: write",fn)) }

  fwrite(&collisions,sizeof(collisions),1,f);
  fwrite(&rdf,sizeof(rdf),1,f);
  fwrite(rdf.blockhist,sizeof(rdf.blockhist[0]),rdf.nhist,f);
  fwrite(rdf.tothist,sizeof(rdf.tothist[0]),rdf.nhist,f);
#ifdef SWPERT
  fwrite(rdf.swsum,sizeof(rdf.swsum[0]),rdf.nsw,f);
  fwrite(rdf.swpow,sizeof(rdf.swpow[0]),rdf.nsw,f);
#endif
  loopto (it,it0,it1) {
    struct event_s *l;

    looplist (l,it==it1?longt:tlist[it])
      if (l->A>UPDATE && l->A<UPDATE+NUPDATE) {
        struct { int iupd; real t; } s;
        
        s.iupd=l->A-UPDATE;
        s.t=l->t;
        if (verbose)
	 prt("%s pending event %s t=%g",
	     l->A==STOP?"skipping":"saving",
	     eventname[s.iupd],s.t);
        if (l->A==STOP) stop=s.t;
        else fwrite(&s,sizeof(s),1,f); } }

  fclose(f);
  time(&ttt);
  prt_("%s saved (stop=%s) @ %s", fn,
      stop<0?"never":string("%g",stop), ctime(&ttt));
  if (out!=stdout) fprintf(stderr,"%s saved (to stop=%g) @ %s",fn,stop,ctime(&ttt));

  fflush(out);
}

void loadall(char *fn,int init) /********************************** loadall */
/* 
  reading configuration, rescaling to new units if necessary
  if init==0 reading rdf + pending events
*/
{
  FILE *f=fopen(fn,"rb");
  int Nfile,i;
  real Lfile,sigmafile,ratioL,ratiosigma;
#ifdef PS
  real vunitfile,ratiov;
#endif    

  if (!f) ERROR(("open %s",fn))
  fread(&Nfile,sizeof(Nfile),1,f);
  if (Nfile!=N) ERROR(("%s: N=%d read, N=%d expected",fn,Nfile,N))
  fread(&Lfile,sizeof(L),1,f);
  fread(&t,sizeof(t),1,f);
  fread(&toffset,sizeof(toffset),1,f);

  prt("t=%g toffset=%g loaded%s",
      t,toffset,init==0||init==1?", tlist adjusted":" but ignored (reset to zero)");
  if (init>=2)
    t=toffset=0;
  else {
    int idt=t;
    
    it0+=idt; it1+=idt; tlist-=idt; }
    
  fread(&sigmafile,sizeof(sigmafile),1,f);

  ratioL=L/Lfile;
  ratiosigma=sigma/sigmafile;

  put3(Lfile,ratioL,ratiosigma)

  if (ratioL!=1. || ratiosigma!=1.) {
    prt("NOTE: rescaling configuration %g times\n\
      (this may be caused by ncell change)",ratioL);
    if (init==0||init==1) ERROR(("hsmd bug: rescaling not allowed for init=0,1")) }

  if (ratioL<ratiosigma*(1-EPS))
    WARNING(("shrink ratio=%.14g: overlap danger",ratioL/ratiosigma))

#ifdef PS
  fread(&novl,sizeof(novl),1,f);
  prt("%d overlaps loaded",novl);
  fread(&vunitfile,sizeof(vunitfile),1,f);
  ratiov=vunitfile/vunit;
  if (ratiov!=1.) {
    prt("NOTE: rescaling velocities %g times\n\
      (this may be caused by slice change)",ratiov);
    if (init==0||init==1) ERROR(("psmd bug: rescaling velocities not allowed for init=0,1")) }
#endif 
  
  loop (i,0,N) {
    if (1!=fread(&cfg[i].r,sizeof(vector),1,f)) ERROR(("%s: unexpectead EOF"))
    VO(cfg[i].r,*=ratioL)
    if (1!=fread(&cfg[i].v,sizeof(vector),1,f)) ERROR(("%s: unexpectead EOF")) 
#ifdef PS
    VO(cfg[i].v,*=ratiov)
#endif    
    cfg[i].t=t; }
  
  
  if (init==0) {
    int nhist=rdf.nhist;
#ifdef SWPERT
    int nsw=rdf.nsw;
    int *swindex=rdf.swindex;
    int *swsum=rdf.swsum;
    double (*swpow)[SWPERT]=rdf.swpow;
#endif
    int *blockhist=rdf.blockhist;
    real *tothist=rdf.tothist;
    struct { int iupd; real t; } s;

    fread(&collisions,sizeof(collisions),1,f);
  
    if (fread(&rdf,sizeof(rdf),1,f)!=1) ERROR(("read rdf"))
    rdf.blockhist=blockhist;
    rdf.tothist=tothist;

    if (nhist!=rdf.nhist)
      ERROR(("rdf.nhist=%d loaded != rdf.nhist=%d specified",rdf.nhist,nhist))

    if (fread(rdf.blockhist,sizeof(rdf.blockhist[0]),rdf.nhist,f)!=rdf.nhist)
      ERROR(("read rdf.blockhist"))

    if (fread(rdf.tothist,sizeof(rdf.tothist[0]),rdf.nhist,f)!=rdf.nhist)
      ERROR(("read rdf.tothist"))

#ifdef SWPERT
    rdf.swindex=swindex;
    rdf.swsum=swsum;
    rdf.swpow=swpow;
    if (rdf.nsw!=nsw)
      ERROR(("rdf.nsw=%d loaded != rdf.nsw=%d specified",rdf.nsw,nsw))
    if (fread(rdf.swsum,sizeof(rdf.swsum[0]),rdf.nsw,f)!=rdf.nsw)
      ERROR(("read rdf.swsum"))
    if (fread(rdf.swpow,sizeof(rdf.swpow[0]),rdf.nsw,f)!=rdf.nsw)
      ERROR(("read rdf.swpow"))
#endif

    while (fread(&s,sizeof(s),1,f)==1) {
      struct event_s *l;

      alloconezero(l);
      l->A=UPDATE+s.iupd;
      l->t=s.t;
      prt("pending event %s t=%g loaded",eventname[s.iupd],s.t);
      scheduleevent(s.t-t,l); }
    }

  fclose(f); 
}

FILE *plb=NULL;
FILE *cp=NULL;

void openplb(char *fn,int init) /*********************************** openplb */
{
  float hdr[2];

  if (init==0||init==1) {
    long pos;

    plb=fopen(fn,"rb");
    if (!plb) ERROR(("cannot append %s",fn))
    if (2!=fread(hdr,4,2,plb)) ERROR(("appending %s: no header",fn))
    if (hdr[0]!=N)
      ERROR(("appending %s: N=%.0f in file (%d expected)",fn,hdr[0],N))
    if (hdr[1]!=(float)userL)
      WARNING(("appending %s: L=%.8f in file (%.8f expected)",fn,hdr[0],userL)) 

    if (fseek(plb,0,2)) ERROR(("%s seek",fn))
    pos=ftell(plb);
    prt("appending %s: size=%ld",fn,pos);
    if ((pos-8)%(4*DIM*N))
      ERROR(("appending %s: fractional last frame (cut to %ld)",fn,
            8+(pos-8)/(4*DIM*N)*(4*DIM*N)) )
    fclose(plb);
    plb=fopen(fn,"ab"); }

  else {
    FILE *mol=fopen(Fn("mol"),"wt"),*gol=fopen(Fn("gol"),"wt");
    int i;

    plb=fopen(fn,"wb");
    if (!plb) ERROR(("write %s",fn))

    hdr[0]=N; hdr[1]=(float)userL;

    fwrite(hdr,4,2,plb);

    fprintf(mol,"HS pure\n\
\n\
parameter_set=n.a.\n\
number_of_atoms=%d\n\
atoms\n\
! i  atom-id  a-type charge chir nbonds bound_atoms\n",N);
    fprintf(gol,"!sphere#1\n\
! HS mix\n\
%d\n",N);
    loop(i,0,N) {
      fprintf(mol,"%3d %d-HS HS 0 0 0\n",i,i);
      fprintf(gol,"%c %f\n","YCWMGRB"[i%7],0.5); }
    fclose(mol);
    fclose(gol); }
}

FILE *cfgf=NULL;

void opencfg(char *fn,int init) /*********************************** openplb */
{
  if (init==0||init==1) {
    long pos;
    double Lfile;
    int Nfile;

    cfgf=fopen(fn,"rb");
    if (!cfgf) ERROR(("cannot append %s",fn))
    fread(&Nfile,sizeof(Nfile),1,cfgf);
    fread(&Nfile,sizeof(Nfile),1,cfgf);
    if (Nfile!=N) ERROR(("%s: N=%d read, N=%d expected",fn,Nfile,N))
    fread(&Lfile,sizeof(Lfile),1,cfgf);
    if (fabs(L/sigma/Lfile-1)>1e-9)
      ERROR(("%s: L=%g read, L=%g expected",fn,Lfile,L))

    if (fseek(cfgf,0,2)) ERROR(("%s seek",fn))
    pos=ftell(cfgf);
    prt("appending %s: size=%ld",fn,pos);
    cfgf=fopen(fn,"ab"); }

  else {
    double LL=L/sigma;

    cfgf=fopen(fn,"wb");
    if (!cfgf) ERROR(("write %s",fn))
    fwrite(&N,sizeof(N),1,cfgf);
    fwrite(&N,sizeof(N),1,cfgf);
    fwrite(&LL,8,1,cfgf); }
}

#define CPmark ((float)-1.755645e+37)
#define NCP 5

void opencp(char *fn,int init) /************************************* opencp */
{
  float rec[NCP];
  long pos;

  if (init==0||init==1) {
    cp=fopen(fn,"rb");
    if (!cp) ERROR(("%s missing or cannot open (for init=0 or 1)",fn))
    if (fseek(cp,0,2)) ERROR(("%s seek",fn))
    pos=ftell(cp);
    if (pos%(NCP*4))
      ERROR(("appending cp: fractional last record (cut to %ld)",
            pos/(NCP*4)*(NCP*4)) )
    fclose(cp);
    cp=fopen(fn,"ab"); }

  else {
    cp=fopen(fn,"wb");

    rec[0]=CPmark; *(int*)(rec+1)=NCP;
#ifdef PS   
    copy(rec+2,"Epotupd Zvir",12);
#else   
    copy(rec+2,"rateg1  upd ",12);
#endif
    fwrite(rec,4,NCP,cp); }
}

void writecp(real A,real B,real C,real D,real E) /***************** writecp */
{
  float rec[NCP];

  rec[0]=A;
  rec[1]=B;
  rec[2]=C;
  rec[3]=D;
  rec[4]=E;
  
  fwrite(rec,4,NCP,cp);
}

void closecp(void) /*********************************************** closecp */
{
  float rec[NCP];
  time_t ttt;

  time(&ttt);

  rec[0]=CPmark;
  copy(rec+1,ctime(&ttt)+4,16);
  
  fwrite(rec,4,NCP,cp);
}

void writeplb(void) /********************************************* writeplb */
/* 
  write one frame to the playback file 
*/
{
  int i;
  float fr[3];
  real isig=1./sigma;

  if (!plb) {
    ERROR(("plb not opened"))
    return; }

  syncHS();

  loop (i,0,N) {
    VV(fr,=isig*cfg[i].r)
    fwrite(fr,4,3,plb); } 

  fflush(plb);
}

void writecfg(void) /********************************************* writeplb */
/* 
  write one frame to the cfg file (double)
*/
{
  int i;
  vector dr;
  real isig=1./sigma;

  if (!cfgf) {
    ERROR(("cfgf not opened - why?"))
    return; }

  syncHS();

  loop (i,0,N) {
    VV(dr,=isig*cfg[i].r)
    fwrite(dr,sizeof(dr),1,cfgf); } 

  fflush(cfgf);
}
