#ifndef PS
double TMC=0.4;

int MC(int init) /******************************************************* MC */
/*
   MC with soft repulsive potential, init sweeps
   Neighbor list used
   Inefficient for low densities
*/
{
  int i,n,ovl=0,k,isw;
  static double d;
  vector newr;
  double Unew,Uold,rrnew,rrold,x,rgq,ar,rgqmin;
  int accept=0;

  struct nn_s {
    /* neighbor list */
    struct nn_s *next;
    int i; /* the neighbor */ } **nbr,*one;

  if (!d) d=sigma*.1; /* quite short -- dense system assumed */

  x=sigma+init*d*2;
  rgq=x*x;
  rgqmin=rgq;

  /* create neighbor list */
  allocarrayzero(nbr,Ntot);
  loop (n,0,Ntot) {
    int nal=0;

    loop (i,0,n) {
      rrnew=0;
      loop (k,0,DIM) {
        x=Lh-fabs(Lh-fabs(cfg[i].r[k]-cfg[n].r[k])),rrnew+=x*x;
        if (rrnew>rgq) goto skip; }
      if (rrnew<rgqmin) rgqmin=rrnew;
      nal++;
      allocone(one);
      one->next=nbr[n];
      nbr[n]=one;
      one->i=i;
      allocone(one);
      one->next=nbr[i];
      nbr[i]=one;
      one->i=n;
    skip:; } }

  rgqmin=sqrt(rgqmin/sigmaq);

  loop (isw,0,init) {
    ovl=0;
    loop (n,0,Ntot) {
      Unew=Uold=0;
      VVO(newr,=cfg[n].r,+d*rndcos())
      loop (k,0,DIM) {
        while (newr[k]<0) newr[k]+=L;
        while (newr[k]>L) newr[k]-=L; }

      looplist (one,nbr[n]) {
        rrold=rrnew=0;
        loop (k,0,DIM) {
          x=Lh-fabs(Lh-fabs(cfg[one->i].r[k]-cfg[n].r[k])),rrold+=x*x;
          x=Lh-fabs(Lh-fabs(cfg[one->i].r[k]-newr[k])),rrnew+=x*x; }

        if (rrnew<sigmaq) Unew+=9-8*sqrt(rrnew/sigmaq);
        if (rrold<sigmaq) Uold+=9-8*sqrt(rrold/sigmaq); }

      if (Unew-Uold<-TMC*log(rnd()+1e-33)) {
        VV(cfg[n].r,=newr)
        if (Unew) ovl++;
        accept++; }
      else {
        if (Uold) ovl++; } } }

  ar=(double)accept/(init*Ntot);
  x=-log(ar+1e-33);
  if (x<0.5) x=0.5;
  if (x>3) x=3;
  d/=x;

  prt("minr=%.5f TMC=%.5f ar=%.3f d=%.5f ovl=%d",rgqmin,TMC,ar,d,ovl);

  loop (n,0,Ntot) {
    struct nn_s *next;
    for (one=nbr[n]; one; one=next) {
      next=one->next;
      free(one); } }
  free(nbr);

  return ovl;
}

int MCold(int init) /************************************************* MCold */
/*
   MC with soft repulsive potential, init sweeps
   Inefficient simple code slightly violating microreversibility
*/
{
  int i,n,ovl=0,k,isw;
  static double d;
  vector newr;
  double Unew,Uold,rrnew,rrold,x;

  if (!d) d=sigma*.3;

  loop (isw,0,init) {
    ovl=0;
    loop (n,0,Ntot) {
      Unew=Uold=0;
      VVO(newr,=cfg[n].r,+d*(rnd()-rnd()+rnd()-rnd()))

      loop (k,0,DIM) {
        while (newr[k]<0) newr[k]+=L;
        while (newr[k]>L) newr[k]-=L; }
      loop (i,0,Ntot) if (i!=n) {
        rrold=rrnew=0;
#if DIM==3
        /* DIM=3: optimize a bit */
        x=Lh-fabs(Lh-fabs(cfg[i].r[0]-cfg[n].r[0])),rrold+=x*x;
        x=Lh-fabs(Lh-fabs(cfg[i].r[0]-newr[0])),rrnew+=x*x;
        if (rrold>sigmaq && rrnew>sigmaq) continue;
        x=Lh-fabs(Lh-fabs(cfg[i].r[1]-cfg[n].r[1])),rrold+=x*x;
        x=Lh-fabs(Lh-fabs(cfg[i].r[1]-newr[1])),rrnew+=x*x;
        if (rrold>sigmaq && rrnew>sigmaq) continue;
        x=Lh-fabs(Lh-fabs(cfg[i].r[2]-cfg[n].r[2])),rrold+=x*x;
        x=Lh-fabs(Lh-fabs(cfg[i].r[2]-newr[2])),rrnew+=x*x;
#else
        loop (k,0,DIM) {
          x=Lh-fabs(Lh-fabs(cfg[i].r[k]-cfg[n].r[k])),rrold+=x*x;
          x=Lh-fabs(Lh-fabs(cfg[i].r[k]-newr[k])),rrnew+=x*x; }
#endif

/*.....       s=Sqr(cfg[n].R+cfg[i].R);*/
        if (rrnew<sigmaq) Unew+=9*sigmaq-8*rrnew;
        if (rrold<sigmaq) Uold+=9*sigmaq-8*rrold; }

      if (Unew-Uold<-TMC*log(rnd()+1e-33)) {
        VV(cfg[n].r,=newr)
        if (Unew) ovl++;
        d*=1.003;
        if (d>L) d=L; }
      else {
        if (Uold) ovl++;
        d*=.999; } } }

  put3(TMC,ovl,d)

  return ovl;
}
#endif

double centerdist(double *r) /*********************************** centerdist */
{
  return sqrt(Sqr(r[0]-Lh)+Sqr(r[1]-Lh)+Sqr(r[2]-Lh));
}

void initcfg(int init,int *LN, double rnucleus) /******************* initcfg */
/*
  initial configuration
  init<0: hc or fcc lattice by hexagonal layers.
          Layers order is coded by |init| as decimal digits 1,2,3, e.g.,
          init=-12 = fcc lattice (maybe distorted)
          init=-123 = hc lattice (maybe distorted)
          Layers are (top view - in the direction of z):
             1     1     1     1     1     1
          3     3     3     3     3     3     3
             2     2     2     2     2     2
          1     1     1     1     1     1     1
             3     3     3     3     3     3
          2     2     2     2     2     2     2
             1     1     1     1     1     1
          3     3     3     3     3     3     3
             2     2     2     2     2     2
          1     1     1     1     1     1     1
             3     3     3     3     3     3
          2     2     2     2     2     2     2
             1     1     1     1     1     1
  init=3: random with MC (neighbor-list)
  init=4: random with MC (inefficient for large N)
  init=5: sc
  init=6: bcc
  init=7: fcc
  (see also initcfg() in mc/ils.c)
  init=5,6,7 only:
    LN: # of elementary cells/box side (0 -> min.possible=min.vacancies)
    rnucleus > 0: size of nucleus (crystal without vacancies) in "fluid"
    rnucleus < 0: size of "fluid: inclusion
                  ("fluid"=crystal with vacancies to reach given # of HS)
  examples of input data:
    ! CRYSTAL NUCLEUS
    rhos=1.041 rhof=0.943              ! estimated solid/fluid densities
    init=7                             ! fcc lattice
    x=0.4                              ! volume ratio of crystal nucleus
    rho=rhos*x+rhof*(1-x)              ! averaged number density
    n=20
    LN[0]=n LN[1]=n LN[2]=n            ! fcc units/box
    n=n^3*4                            ! # of crystal sites
    V=n/rhos                           ! volume (==> L=V^(1/3))
    N=int(V*rho+0.5)                   ! number of spheres
    rnucleus=(3*x*n/pi/rhos/4)^(1/3)   ! nucleus radius

    ! INCLUSION IN CRYSTAL
    n=30
    LN[0]=n LN[1]=n LN[2]=n            ! fcc units/box
    n=n^3*4                            ! # of crystal sites
    rhos=1.038 rhof=0.943              ! estimated solid/fluid densities
    x=0.6                              ! vol. ratio of crystal (inclusion=1-x)
    rho=rhos*x+rhof*(1-x)              ! averaged number density
    V=n/rhos                           ! volume (==> L=V^(1/3))
    N=int(V*rho+0.5)                   ! number of spheres
    rnucleus=-(3*(1-x)*n/pi/rhos/4)^(1/3) ! inclusion radius
    init=7                             ! fcc lattice

x=0.330
rhos=1.041
rhof=0.943
rho=rhos*x+rhof*(1-x)
n= 85*98*104
V=n/rhos
N=int(V*rho+0.5)
LN[0]=85 LN[1]=98 LN[2]=104

init=-12

rnucleus=(3*x*n/pi/rhos/4)^(1/3)


*/
{
  int i,sym=1,Nlatt,ilatt,nMC=0;
  real a=0,nn=0,shift;
  vector *r,*cell=NULL;
  int patlen=0,i1,i2;
  char pattern[12],lastpat='1';

  prt_("initial lattice: ");
  switch (init<0?-1:init) {
    case 3: prt("random cfg + soft-sphere MC, neighbor-list"); break;
    case 4: prt("random cfg + soft-sphere MC, simple code"); break;
    case 5: prt("simple cubic"); break;
    case 6: prt("body centered cubic"); sym=2; break;
    case 7: prt("face centered cubic"); sym=4; break;
    case -1:
      prt("honeycomb/fcc layers, layer pattern=%d (in z-direction)",-init); sym=1;
      sprintf(pattern,"%d",-init);
      patlen=strlen(pattern);
      if (patlen<2)
        ERROR(("pattern of layers too short"))
      if (LN[1]&1)
        WARNING(("odd LN[1] may cause overlaps"))
      i1=(LN[2]-1)%patlen; i2=(LN[2]-2)%patlen;
      lastpat=pattern[i1];
      //     fprintf(stderr,"%c%c%c\n",pattern[0],pattern[i1],pattern[i2]) ;
      if (pattern[0]==pattern[i1] || pattern[i1]==pattern[i2]) {
        if (pattern[0]==pattern[i2]) {
         if (pattern[0]=='1') lastpat='2';
         else lastpat='1'; }
        else
         lastpat='1'+'2'+'3'-pattern[0]-pattern[i2];
        WARNING(("last layer changed from %c to %c to prevent overlaps",
                pattern[i1],lastpat)) }
      break;
    default: {
      /*
        file SIMNAME.### should have ### HS in a unit box
        no overlap check!!!
        can be obtained e.g. by hsmc (with SLOWLY increasing rho)
        and checking RDF whether all overlaps are removed
        then, the .plb file is converted to .pla and rescaled to interval [0,1)
      */
/*.....     char *fn=string("cell.%d",init);*/
      char *fn=Fn(string("%d",init));
      FILE *f=fopen(fn,"rt");
      char line[128];
      int j;

      prt("repeating external cubic cell \"%s\"",fn);
      if (!f) ERROR(("no %s",fn))
      sym=init;
      allocarray(cell,sym);
      loop (j,0,sym) {
        if (!fgets(line,128,f)) ERROR(("not enough lines in %s",fn))
        if (sscanf(line,"%lf%lf%lf",cell[j],cell[j]+1,cell[j]+2)!=3)
         ERROR(("x y z expected in %s",fn)) }
      fclose(f); }
    }

  if (init==3 || init==4)
    Nlatt=N;
  else if (init<0)
    Nlatt=LN[0]*LN[1]*LN[2];
  else {
    if (LN[0]==0) {
      while ( (Nlatt=Cub(LN[0])*sym) <N) LN[0]++; }
    if ( (Nlatt=Cub(LN[0])*sym) <N)
      ERROR(("more atoms than lattice sites"))
    if (Nlatt != N)
      WARNING(("partially filled lattice: %d spheres in %d lattice sites",N,Nlatt))
    nn=a=L/LN[0];
    prt("lattice unit a=%g = %g sigma",a,a/sigma); }

  if (Nlatt<=0) ERROR(("bad # of lattice sites or perhaps LN[]"))
  allocarray(r,Nlatt);
  if (Nlatt<N) ERROR(("more atoms than lattice sites"))

  if (sym>1) shift=a/4; else shift=a/2;

  if (sym==2) nn*=sqrt(2);
  else if (sym==4) nn*=sqrt(3);

  if (abs(init-6)<2) {
    double q=nn/sigma; /* nearest-neighbor in the units of sigma, sc */

    if (init==6) q/=sqrt(8./3);
    if (init==7) q/=sqrt(6.);
    prt("lattice: nearest neighbor = %g sigma",q);
    if (q<1)
      ERROR(("overlap in initial lattice"))
    if (q<1.01)
      WARNING(("nn=%g sigma too short, slow equilibrating expected",q)) }

  loop (i,0,Nlatt) {
    switch (init<0?-1:init) {
      case 3:
      case 4:
        VO(r[i],=L*rnd()) break;
      case 6:
        /* bcc lattice */
        if (i&1) {
          VV(r[i],=a/2+r[i-1])
          break; }
      case 5:
        /* sc lattice */
      sc:
        r[i][0]=((i/sym)/(LN[0]*LN[0]))*a+shift;
        r[i][1]=(((i/sym)/LN[0])%LN[0])*a+shift;
        r[i][2]=((i/sym)%LN[0])*a+shift;
        break;
      case 7:
        /* fcc lattice */
        if (i&3) {
          VV(r[i],=a/2+r[i&~3])
          r[i][(i&3)-1]-=a/2;
          break; }
        else
          goto sc;
      case -1: {
        /* layers of hc/fcc lattice */
        int x=i%LN[0];
        int y=(i/LN[0])%LN[1];
        int z=i/(LN[0]*LN[1]);
        int pat=pattern[z%patlen];

        if (z==LN[2]-1) pat=lastpat;

        r[i][0]=L/LN[0]*(x+y/2.);
        r[i][1]=L/LN[1]*y; // squezed sqrt(0.75) times
        r[i][2]=L/LN[2]*z; // squezed sqrt(2/3) times
        switch (pat) {
          case '2': r[i][1]+=L/LN[1]/0.75; break;
          case '3': r[i][1]+=L/LN[1]/1.5; }
        loop (z,0,DIM) r[i][z]=fmod(r[i][z],L);
        break; }

      default:
        /* external cell */
        r[i][0]=(((i/sym)/(LN[0]*LN[0]))+cell[i%sym][0])*a;
        r[i][1]=((((i/sym)/LN[0])%LN[0])+cell[i%sym][1])*a;
        r[i][2]=(((i/sym)%LN[0])+cell[i%sym][2])*a;
        break; } }

  if (fabs(rnucleus)>=L/2)
    WARNING(("nucleus=%g bigger than box/2=%g",rnucleus,L/2))

  if (rnucleus>0) {
    loop (i,0,Nlatt-N) {
      do ilatt=irnd(Nlatt);
      while (r[ilatt][0]==-999 || centerdist(r[ilatt])<rnucleus);
      r[ilatt][0]=-999; } }
  else if (rnucleus<0) {
    loop (i,0,Nlatt-N) {
      do ilatt=irnd(Nlatt);
      while (r[ilatt][0]==-999 || centerdist(r[ilatt])>-rnucleus);
      r[ilatt][0]=-999; } }
  else {
    loop (i,0,Nlatt-N) {
      do ilatt=irnd(Nlatt); while (r[ilatt][0]==-999);
      r[ilatt][0]=-999; } }

  i=0;
  loop (ilatt,0,Nlatt) if (r[ilatt][0]>-999) {
    cfg[i].t=t;
    VV(cfg[i].r,=r[ilatt])
    VO(cfg[i].v,=rndgauss()) /* HSMD: rescaled later, PS,SW: changed later */
    i++; }
  if (i!=N) ERROR(("initcfg: internal (i=%d N=%d)",i,N))

  if (cell) free(cell);

  if (0) {
    prt("%d %g %g",Ntot,L,sigma);
    loop (i,0,Ntot) prt("%g %g %g",VARG(cfg[i].r));
    fflush(out); }

#ifndef PS
 /* recommended Tprotocol=0.1 for rho>1 */
  if (init==3)
    while (prt_("%4d: ",nMC+=8),MC(8)) TMC*=1-Tprotocol*sqrt(TMC);

  if (init<0) {
    /* check overlaps (they would cause crash!) */
    TMC=0;
    if (MC(1)) {
      WARNING(("overlap(s) found, MC forced"))
      TMC=0.2;
      while (prt_("%4d: ",nMC+=8),MC(8)) TMC*=1-Tprotocol*sqrt(TMC); } }

  if (init==4)
    while (prt_("%4d: ",nMC+=3),MCold(3)) TMC*=1-Tprotocol*sqrt(TMC);
#endif

  free(r);
}

void makehash(void) /********************************************* makehash */
/*
   assigns hash values to spheres
   used to implement a large sparse pair table
*/
{
  int i;

 // prt("makehash");

  loop (i,0,Ntot)
    cfg[i].hash=(int)((double)nhash*i/(double)Ntot);

  /* randomize the hash table */
  loop (i,0,Ntot*3) {
    int j=irnd(Ntot-2)+1;
    cfg[j].hash=(cfg[j-1].hash+irnd(cfg[j+1].hash-cfg[j-1].hash-1)+1+nhash)%nhash; }

  /*..... loop (i,0,Ntot) prt("%d %d ~",i,cfg[i].hash);*/
}

void buildcells(void) /***************************************** buildcells */
/*
   builds the cell array box[][][]
*/
{
  int i,k;

  loop (i,0,N) {
    loop (k,0,DIM) {
      cfg[i].ir[k]=(int)cfg[i].r[k];
      if (cfg[i].ir[k]<0 || cfg[i].ir[k]>=ncell)
        ERROR(("initial HS %d[%d]:%d %g out of box",
              i,k,cfg[i].ir[k],cfg[i].r[k])) }
    cfg[i].cell=&box[cfg[i].ir[0]][cfg[i].ir[1]][cfg[i].ir[2]];
    cfg[i].next=*cfg[i].cell;
    *cfg[i].cell=&cfg[i]; }
}

void makelists(void) /******************************************* makelists */
/*
   building the tables and lists for the whole configuration
*/
{
  intvector ir;
  struct HS_s *B;

  allocarrayzero(pair,nhash);

  loop (ir[0],0,ncell)
    loop (ir[1],0,ncell)
      loop (ir[2],0,ncell)
        looplist (B,box[ir[0]][ir[1]][ir[2]]) {
          allocone(B->update);
         B->update->A=UPDATE;
         B->update->B=B;
         scheduleevent(0,B->update); }
}
