/* \make hsmd
 */
real RSUBCELL=NSUBCELL-EPS;

static intvector *auxnbr;

struct nbr_s {
  int fromx,tox; /* C-style, i.e. excl. tox */
  struct nbrx_s {
    int fromy,toy;
    struct nbry_s {
      int fromz,toz; } *y;
    } *x;
  } nbrs[NSUBCELL][NSUBCELL][NSUBCELL], /* for moving, upto cutoff */
    rdfnbrs[NSUBCELL][NSUBCELL][NSUBCELL]; /* for rdf, upto rdf.cutoff */

static int cellmindistq(intvector sub,intvector ir) /********* cellmindistq */
/*
  minimum distance^2 of subcell sub from cell ir
*/
{
 intvector dr;
 int k;

 loop (k,0,DIM)
   if (ir[k]<0) dr[k]=sub[k]-NSUBCELL*(ir[k]+1);
   else if (ir[k]>0) dr[k]=NSUBCELL*ir[k]-sub[k]-1;
   else dr[k]=0;

 return SQR(dr);
}

                                                               /*** makenbr */
void makenbrs(struct nbr_s nbrs[NSUBCELL][NSUBCELL][NSUBCELL],
              real cut,int half)
/*
  makes array of neighboring cells for NSUBCELL^s subcells
  if half then one half of them (each pair appears only once), as for rdf
*/
{
 int pass,icut=cut+3;
 intvector ir,sub;
 int nnbr=0,icutq=0x7fffffff,sumnnbr=0;
 real cutq=Sqr(cut*NSUBCELL);

 loop (sub[0],0,NSUBCELL)
   loop (sub[1],0,NSUBCELL)
     loop (sub[2],0,NSUBCELL) {

       loop (pass,0,2) {
         if (pass) allocarray(auxnbr,nnbr);
         nnbr=0;
         loopto (ir[0],-icut,icut)
           loopto (ir[1],-icut,icut)
             loopto (ir[2],-icut,icut) {
               int iq;

               iq=cellmindistq(sub,ir);

               if (half && ir[0]==0 && ir[1]==0 && ir[2]==0) goto done;
	       /* center not included */

/*.....               if (ir[0]==0 && ir[1]==0 && ir[2]==0) if (half) goto done; else continue;*/

               if (iq<cutq) {
                 if (pass)
		   /* there is another (optimized) algorithm in measurerdf() */
                   VV(auxnbr[nnbr],=(!half)*ncell+ir)
                 nnbr++; }
               else 
		 Min(icutq,iq) }

        done:; 
         }

/*.....       prt("neighboring cells in cutoff %g, sub %d %d %d: half=%d, nnbr=%d", cut,sub[0],sub[1],sub[2],half,nnbr);*/
       if (nnbr>10000*(1+half*9)) WARNING(("too many neighboring cells"))
#if defined(DEBUG) && NSUBCELL<4
       if (!half) {
         FILE *plb=fopen(string("nbr%d%d%d.plb",sub[0],sub[1],sub[2]),"wb");
         FILE *mol=fopen(string("nbr%d%d%d.mol",sub[0],sub[1],sub[2]),"wt");
         FILE *gol=fopen(string("nbr%d%d%d.gol",sub[0],sub[1],sub[2]),"wt");
         float f[3];
         int i;
   
         f[0]=nnbr; f[1]=0;
         fwrite(f,4,2,plb);
         fprintf(mol,"nbr\n\nnumber_of_atoms = %d\n\natoms\n",nnbr);
         fprintf(gol,"!sphere#1\n!\n%d\n",nnbr);

         loop (i,0,nnbr) {
           VV(f,=auxnbr[i])
           fwrite(f,4,3,plb);
           fprintf(mol,"%d HA HA 0 0 0 0\n",i);
           fprintf(gol,"WHITE 0.5\n"); }
         fclose(plb);
         fclose(mol);
         fclose(gol); }
#endif
       sumnnbr+=nnbr;

       /* this is cumbersome, but auxnbr-style code was first ...*/
       {
         int inbr;
         int x,fromx=ncell,tox=-ncell;
         struct nbr_s *nbr=&nbrs[sub[0]][sub[1]][sub[2]];

         loop (inbr,0,nnbr) {
           Min(fromx,auxnbr[inbr][0])
           Max(tox,auxnbr[inbr][0]) }

         nbr->fromx=fromx;
         nbr->tox=++tox;
         allocarray(nbr->x,tox-fromx);
         nbr->x-=fromx;

         loop (x,fromx,tox) {
           int y,fromy=ncell,toy=-ncell;

           loop (inbr,0,nnbr) if (auxnbr[inbr][0]==x) {
             Min(fromy,auxnbr[inbr][1])
             Max(toy,auxnbr[inbr][1]) }
           
           nbr->x[x].fromy=fromy;
           nbr->x[x].toy=++toy;
           allocarray(nbr->x[x].y,nbr->x[x].toy-fromy);
           nbr->x[x].y-=fromy;

           loop (y,fromy,toy) {
             int fromz=ncell,toz=-ncell;

             loop (inbr,0,nnbr) if (auxnbr[inbr][0]==x && auxnbr[inbr][1]==y) {
               Min(fromz,auxnbr[inbr][2])
               Max(toz,auxnbr[inbr][2]) }
             nbr->x[x].y[y].fromz=fromz;
             nbr->x[x].y[y].toz=toz+1; }
           } /*x */

         free(auxnbr);
       }
       }
 prt("%s: average # of neighboring cells=%g",half?"rdf":"collide",
     (real)sumnnbr/Cub(NSUBCELL));

 if (!half) {
   cutoff=sqrt(icutq)/RSUBCELL;
   prt("with the same number of cells, cutoff changed to %g (prog units:%g)",
       cutoff/sigma,cutoff); }

}
